@@ -193,7 +193,7 @@
        (eval-expr
 	(lambda (tree)
 	  (case (car tree)
-	    ((fixed) (string->number (tx1 tree)))
+	    ((fixed) (string->number (cnumstr->scm (tx1 tree))))
 	    ((char) (char->integer (tx1 tree)))
 	    ((defined) (if (assoc-ref dict (tx1 tree)) 1 0))
 	    ((pre-inc post-inc) (1+ (ev1 tree)))
@@ -271,12 +271,24 @@
 	((('ident . rval) ('ident . lval) . rest)
 	 (iter stl chl (string-append " " rval) (cdr tkl)))
 
-	(((key . val) . rest)
+	((('string . val) . rest)
+	 (iter stl (cons #\" chl) val (cons #\" rest)))
+
+	((('ident . val) . rest)
+	 (iter stl chl val rest))
+
+	((('arg . val) . rest)
+	 (iter stl chl val rest))
+
+	((('defined . val) . rest)
 	 (iter stl chl val rest))
 
 	(('space . rest)
 	 (iter stl (cons #\space chl) nxt rest))
 
+	((asis . rest)
+	 (iter stl chl asis rest))
+
 	(otherwise
 	 (error "no match" tkl)))))))
 
@@ -294,26 +306,21 @@
 
   ;; We just scanned "defined", now need to scan the arg to inhibit expansion.
   ;; For example, we have scanned "defined"; we now scan "(FOO)" or "FOO", and
-  ;; return "defined(FOO)".  We use ec (end-char) as terminal char:
-  ;; #\) if starts with #( or #\nul if other.
+  ;; return "defined(FOO)" or "defined FOO".
   (define (scan-defined-arg)
-    (let* ((ch (skip-il-ws (read-char)))
-	   (ec (if (char=? ch #\() #\) #\null)))
-      (let iter ((chl '(#\())
-		 (ec ec)
-		 (ch (if (char=? ec #\)) (skip-il-ws (read-char)) ch)))
+    (let* ((ch (skip-il-ws (read-char))) (no-ec (not (char=? ch #\())))
+      (let iter ((chl (list ch)) (ch (skip-il-ws (read-char))))
 	(cond
 	 ((eof-object? ch)
-	  (if (char=? ec #\null)
-	      (string-append "defined" (list->string (reverse (cons #\) chl))))
+	  (if no-ec
+	      (string-append "defined " (list->string (reverse chl)))
 	      (cpp-err "illegal argument to `defined'")))
 	 ((char-set-contains? c:ir ch)
-	  (iter (cons ch chl) ec (read-char)))
-	 ((char=? ec #\))
-	  (if (char=? #\) (skip-il-ws ch))
-	      (string-append "defined" (list->string (reverse (cons #\) chl))))
-	      (cpp-err "garbage in argument to `defined'")))
-	 ((char=? ec #\null) ;; past identifier
+	  (iter (cons ch chl) (read-char)))
+	 (no-ec
+	  (unread-char ch)
+	  (string-append "defined " (list->string (reverse chl))))
+	 ((char=? #\) (skip-il-ws ch))
 	  (string-append "defined" (list->string (reverse (cons #\) chl)))))
 	 (else
 	  (cpp-err "illegal argument to  `defined'"))))))
@@ -369,12 +376,12 @@
 	       (aval			; arg ref
 		(iter (acons 'arg aval tkl) lvl (read-char)))
 	       ((string? rval)		; cpp repl
-		(iter (acons 'string rval tkl) lvl (read-char)))
+		(iter (cons rval tkl) lvl (read-char)))
 	       ((pair? rval)		; cpp macro
 		(let* ((argl (car rval)) (text (cdr rval))
 		       (argd (collect-args argl argd dict used))
 		       (newl (expand-cpp-repl text argd dict (cons iden used))))
-		  (iter (acons 'string newl tkl) lvl (read-char))))
+		  (iter (cons newl tkl) lvl (read-char))))
 	       (else			; normal identifier
 		(iter (acons 'ident iden tkl) lvl (read-char))))))))
      (else
